(*******************************************************************
 *
 *      Component: TRpnGauge v1.0
 *         Author: Richard Philip Nguyen (rpngu3@hotbot.com)
 *           Date: 1 Feb 1999
 *      CopyRight: Richard Philip Nguyen
 *                 Portions CopyRighted by Borland/Inprise Intl.
 *         Status: Freeware
 *
 *    Description: Nice progress bar found in WinRAR95
 *
 *          Legal: You are free to use this component in personal and
 *                 commercial applications as long as due credit is given.
 *
 *                 If you change the source code, I ask you to send me
 *                 a copy of your modified code so I can update the
 *                 the component for the next release.
 *
 *        History: 1 Feb 1999 - First release of the component
 *
 *******************************************************************)

{$R RpnGauge.dcr}

unit RpnGauge;

interface

uses SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, StdCtrls;

type

  TRpnGauge = class(TGraphicControl)
  private
    FMinValue: Longint;
    FMaxValue: Longint;
    FCurValue: Longint;
    FForeColor: TColor;
    FBackColor: TColor;
    procedure SetForeColor(Value: TColor);
    procedure SetBackColor(Value: TColor);
    procedure SetMinValue(Value: Longint);
    procedure SetMaxValue(Value: Longint);
    procedure SetProgress(Value: Longint);
    function  GetPercentDone: Longint;
    procedure DrawOutline(TheCanvas:TCanvas);
    procedure DrawBar(TheCanvas : TCanvas);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure AddProgress(Value: Longint);
    property PercentDone: Longint read GetPercentDone;
  published
    property Align;
    property Color;
    property Enabled;
    property Font;
    property ForeColor: TColor read FForeColor write SetForeColor default clBlack;
    property BackColor: TColor read FBackColor write SetBackColor default clWhite;
    property MinValue: Longint read FMinValue write SetMinValue default 0;
    property MaxValue: Longint read FMaxValue write SetMaxValue default 100;
    property ParentColor;
    property ParentShowHint;
    property PopupMenu;
    property Progress: Longint read FCurValue write SetProgress;
    property ShowHint;
    property Visible;
  end;

  procedure Register;

implementation

uses Consts;

{ This function solves for x in the equation "x is y% of z". }
function SolveForX(Y, Z: Longint): Longint;
(*****************************************)
begin
  Result := Trunc( Z * (Y * 0.01) );
end;

{ This function solves for y in the equation "x is y% of z". }
function SolveForY(X, Z: Longint): Longint;
(*****************************************)
begin
  if Z = 0 then Result := 0
  else Result := Trunc( (X * 100.0) / Z );
end;

{ TRpnGauge }

constructor TRpnGauge.Create(AOwner: TComponent);
(***********************************************)
begin
  inherited Create(AOwner);
  ControlStyle := [csOpaque];
  { default values }
  FMinValue    := 0;
  FMaxValue    := 100;
  FCurValue    := 0;
  FForeColor   := $00C7C7C7;
  FBackColor   := clGray;
  Width        := 145;
  Height       := 26;
end;

function TRpnGauge.GetPercentDone: Longint;
(*****************************************)
begin
  Result := SolveForY(FCurValue - FMinValue, FMaxValue - FMinValue);
end;

procedure TRpnGauge.DrawOutline(TheCanvas:TCanvas);
(*************************************************)
begin
  with TheCanvas do begin
    //main frame
    Pen.Color   := FForeColor;
    Pen.Width   := 2;
    Brush.Color := FBackColor;
    Brush.Style := bsSolid;
    Rectangle(0,0,Width-2,Height-2);

    //shadow
    Pen.Color   := FBackColor;
    MoveTo(2, Height - 1);
    LineTo(Width - 1, Height-1);
    LineTo(Width - 1, 2);

    // draw corners
    Pen.Color   := Color;
    LineTo(Width - 1, 0);
    MoveTo(2, Height-1);
    LineTo(0, Height - 1);
  end;
end;

procedure TRpnGauge.DrawBar(TheCanvas : TCanvas);
(***********************************************)
var
  PercentDone : longint;
  wBar        : longint;
begin
  with TheCanvas do begin
    PercentDone := GetPercentDone;
    wBar := Trunc((Width - 2) * PercentDone / 100);
    if wBar = 0 then exit;

    Brush.Color := FForeColor;
    Brush.Style := bsSolid;
    FillRect(Rect(0, 0, wBar, Height - 2));

    Pen.Color   := clWhite;
    Pen.Width   := 2;
    MoveTo(0, Height - 2);
    LineTo(0, 0);
    LineTo(wBar, 0);

    if PercentDone<100 then
      Pen.Color   := clBlack
    else
      Pen.Color   := FBackColor;

    LineTo(wBar, Height - 2);

    Pen.Color   := FBackColor;
    LineTo(0, Height - 2);

  end;
end;

procedure TRpnGauge.Paint;
(************************)
var
  TheImage : TBitmap;
begin
  with Canvas do begin
    TheImage        := TBitmap.Create;
    TheImage.Height := Height;
    TheImage.Width  := Width;
    DrawOutline(TheImage.Canvas);
    DrawBar(TheImage.Canvas);
    CopyMode := cmSrcCopy;
    Draw(0, 0, TheImage);
    TheImage.Free;
  end;
end;

procedure TRpnGauge.SetForeColor(Value: TColor);
(**********************************************)
begin
  if Value <> FForeColor then
  begin
    FForeColor := Value;
    Refresh;
  end;
end;

procedure TRpnGauge.SetBackColor(Value: TColor);
(**********************************************)
begin
  if Value <> FBackColor then
  begin
    FBackColor := Value;
    Refresh;
  end;
end;

procedure TRpnGauge.SetMinValue(Value: Longint);
(**********************************************)
begin
  if Value <> FMinValue then
  begin
    if Value > FMaxValue then
      if not (csLoading in ComponentState) then
        raise EInvalidOperation.CreateFmt(SOutOfRange, [-MaxInt, FMaxValue - 1]);
    FMinValue := Value;
    if FCurValue < Value then FCurValue := Value;
    Refresh;
  end;
end;

procedure TRpnGauge.SetMaxValue(Value: Longint);
(**********************************************)
begin
  if Value <> FMaxValue then
  begin
    if Value < FMinValue then
      if not (csLoading in ComponentState) then
        raise EInvalidOperation.CreateFmt(SOutOfRange, [FMinValue + 1, MaxInt]);
    FMaxValue := Value;
    if FCurValue > Value then FCurValue := Value;
    Refresh;
  end;
end;

procedure TRpnGauge.SetProgress(Value: Longint);
(**********************************************)
var
  TempPercent: Longint;
begin
  TempPercent := GetPercentDone;  { remember where we were }
  if Value < FMinValue then
    Value := FMinValue
  else if Value > FMaxValue then
    Value := FMaxValue;
  if FCurValue <> Value then
  begin
    FCurValue := Value;
    if TempPercent <> GetPercentDone then { only Refresh if percentage changed }
      Refresh;
  end;
end;

procedure TRpnGauge.AddProgress(Value: Longint);
(**********************************************)
begin
  Progress := FCurValue + Value;
  Refresh;
end;

procedure Register;
(*****************)
begin
  RegisterComponents('Rpn', [TRpnGauge]);
end;

end.
